@@ -78,6 +78,7 @@ function useCompilerDriverLinker(product, inputs) {
 
 function collectLibraryDependencies(product, isDarwin) {
     var publicDeps = {};
+    var privateDeps = {};
     var objects = [];
     var objectByFilePath = {};
     var tagForLinkingAgainstSharedLib = product.cpp.imageFormat === "pe"
@@ -170,6 +171,8 @@ function collectLibraryDependencies(product, isDarwin) {
                 && typeof dep.artifacts[tagForLinkingAgainstSharedLib] !== "undefined";
         if (!isStaticLibrary && !isDynamicLibrary)
             return;
+        if (isBelowIndirectDynamicLib && privateDeps[dep.name])
+            return;
 
         var nextIsBelowIndirectDynamicLib = isBelowIndirectDynamicLib || isDynamicLibrary;
         dep.dependencies.forEach(function(depdep) {
@@ -184,7 +187,8 @@ function collectLibraryDependencies(product, isDarwin) {
         if (isStaticLibrary) {
             if (!isBelowIndirectDynamicLib) {
                 addArtifactFilePaths(dep, "staticlibrary", addPublicFilePath);
-                addExternalLibs(dep);
+                if (product.cpp.importPrivateLibraries)
+                    addExternalLibs(dep);
                 publicDeps[dep.name] = true;
             }
         } else if (isDynamicLibrary) {
@@ -193,6 +197,7 @@ function collectLibraryDependencies(product, isDarwin) {
                 publicDeps[dep.name] = true;
             } else {
                 addArtifactFilePaths(dep, tagForLinkingAgainstSharedLib, addPrivateFilePath);
+                privateDeps[dep.name] = true;
             }
         }
     }
@@ -204,7 +209,6 @@ function collectLibraryDependencies(product, isDarwin) {
     product.dependencies.forEach(traverseDirectDependency);
     addExternalLibs(product);
 
-    var seenRPathLinkDirs = {};
     var result = { libraries: [], rpath_link: [] };
     objects.forEach(
                 function (obj) {
@@ -215,10 +219,7 @@ function collectLibraryDependencies(product, isDarwin) {
                                                 framework: obj.framework });
                     } else {
                         var dirPath = FileInfo.path(obj.filePath);
-                        if (!seenRPathLinkDirs.hasOwnProperty(dirPath)) {
-                            seenRPathLinkDirs[dirPath] = true;
-                            result.rpath_link.push(dirPath);
-                        }
+                        result.rpath_link.push(dirPath);
                     }
                 });
     return result;
@@ -264,8 +265,6 @@ function escapeLinkerFlags(product, inputs, linkerFlags) {
 function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPath) {
     var isDarwin = product.qbs.targetOS.includes("darwin");
     var libraryDependencies = collectLibraryDependencies(product, isDarwin);
-    var frameworks = product.cpp.frameworks;
-    var weakFrameworks = product.cpp.weakFrameworks;
     var rpaths = (product.cpp.useRPaths !== false) ? product.cpp.rpaths : undefined;
     var systemRunPaths = product.cpp.systemRunPaths || [];
     var canonicalSystemRunPaths = systemRunPaths.map(function(p) {
@@ -278,7 +277,7 @@ function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPat
     if (primaryOutput.fileTags.includes("dynamiclibrary")) {
         if (isDarwin) {
             args.push((function () {
-                var tags = ["c", "cpp", "objc", "objcpp", "asm_cpp"];
+                var tags = ["c", "cpp", "cppm", "objc", "objcpp", "asm_cpp"];
                 for (var i = 0; i < tags.length; ++i) {
                     if (linkerPath === product.cpp.compilerPathByLanguage[tags[i]])
                         return "-dynamiclib";
@@ -406,8 +405,8 @@ function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPat
 
     // Flags for library search paths
     var allLibraryPaths = Cpp.collectLibraryPaths(product);
-    if (systemRunPaths.length > 0)
-        allLibraryPaths = allLibraryPaths.filter(isNotSystemRunPath);
+    var builtIns = product.cpp.compilerLibraryPaths
+    allLibraryPaths = allLibraryPaths.filter(function(p) { return !builtIns.includes(p); });
     args = args.concat(allLibraryPaths.map(function(path) { return product.cpp.libraryPathFlag + path }));
 
     escapableLinkerFlags = escapableLinkerFlags.concat(Cpp.collectLinkerScriptPathsArguments(product, inputs));
@@ -432,22 +431,6 @@ function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPat
     args = args.concat(Cpp.collectLinkerObjectPaths(inputs));
     args = args.concat(Cpp.collectResourceObjectPaths(inputs));
 
-    for (i in frameworks) {
-        frameworkExecutablePath = PathTools.frameworkExecutablePath(frameworks[i]);
-        if (FileInfo.isAbsolutePath(frameworkExecutablePath))
-            args.push(frameworkExecutablePath);
-        else
-            args = args.concat(['-framework', frameworks[i]]);
-    }
-
-    for (i in weakFrameworks) {
-        frameworkExecutablePath = PathTools.frameworkExecutablePath(weakFrameworks[i]);
-        if (FileInfo.isAbsolutePath(frameworkExecutablePath))
-            args = args.concat(['-weak_library', frameworkExecutablePath]);
-        else
-            args = args.concat(['-weak_framework', weakFrameworks[i]]);
-    }
-
     var wholeArchiveActive = false;
     var prevLib;
     for (i = 0; i < libraryDependencies.libraries.length; ++i) {
@@ -477,7 +460,10 @@ function linkerFlags(project, product, inputs, outputs, primaryOutput, linkerPat
         if (isDarwin && symbolLinkMode) {
             if (!["lazy", "reexport", "upward", "weak"].includes(symbolLinkMode))
                 throw new Error("unknown value '" + symbolLinkMode + "' for cpp.symbolLinkMode");
+        }
 
+        var supportsLazyMode = Utilities.versionCompare(product.cpp.compilerVersion, "15.0.0") < 0;
+        if (isDarwin && symbolLinkMode && (symbolLinkMode !== "lazy" || supportsLazyMode)) {
             if (FileInfo.isAbsolutePath(lib) || lib.startsWith('@'))
                 escapableLinkerFlags.push("-" + symbolLinkMode + "_library", lib);
             else if (dep.framework)
@@ -582,6 +568,9 @@ function languageTagFromFileExtension(toolchain, fileName) {
     };
     if (!toolchain.includes("clang"))
         m["sx"] = "asm_cpp"; // clang does NOT recognize .sx
+    else
+        m["cppm"] = "cppm";
+
     return m[fileName.substring(i + 1)];
 }
 
@@ -769,12 +758,12 @@ function standardFallbackValueOrDefault(toolchain, compilerVersion, languageVers
     return languageVersion;
 }
 
-function compilerFlags(project, product, input, output, explicitlyDependsOn) {
+function compilerFlags(project, product, outputs, input, output, explicitlyDependsOn) {
     var i;
 
     // Determine which C-language we're compiling
     var tag = ModUtils.fileTagForTargetLanguage(input.fileTags.concat(output.fileTags));
-    if (!["c", "cpp", "objc", "objcpp", "asm_cpp"].includes(tag))
+    if (!["c", "cpp", "cppm", "objc", "objcpp", "asm_cpp"].includes(tag))
         throw ("unsupported source language: " + tag);
 
     var compilerInfo = effectiveCompilerInfo(product.qbs.toolchain,
@@ -794,6 +783,10 @@ function compilerFlags(project, product, input, output, explicitlyDependsOn) {
         args.push('-Os');
     if (opt === 'none')
         args.push('-O0');
+    if (input.cpp.forceUseCxxModules) {
+        if (!product.qbs.toolchain.includes("clang"))
+            args.push('-fmodules-ts')
+    }
 
     var warnings = input.cpp.warningLevel
     if (warnings === 'none')
@@ -805,6 +798,14 @@ function compilerFlags(project, product, input, output, explicitlyDependsOn) {
     if (input.cpp.treatWarningsAsErrors)
         args.push('-Werror');
 
+    var moduleMap = (outputs["modulemap"] || [])[0];
+    if (moduleMap) {
+        const moduleMapperFlag = product.qbs.toolchain.includes("clang")
+            ? "@" // clang uses response file with the list of flags
+            : "-fmodule-mapper="; // gcc uses file with special syntax
+        args.push(moduleMapperFlag + moduleMap.filePath);
+    }
+
     args = args.concat(configFlags(input));
 
     if (!input.qbs.toolchain.includes("qcc"))
@@ -833,7 +834,7 @@ function compilerFlags(project, product, input, output, explicitlyDependsOn) {
 
     var enableExceptions = input.cpp.enableExceptions;
     if (enableExceptions !== undefined) {
-        if (tag === "cpp" || tag === "objcpp")
+        if (tag === "cpp" || tag === "objcpp" || tag === "cppm")
             args.push(enableExceptions ? "-fexceptions" : "-fno-exceptions");
 
         if (tag === "objc" || tag === "objcpp") {
@@ -844,7 +845,7 @@ function compilerFlags(project, product, input, output, explicitlyDependsOn) {
     }
 
     var enableRtti = input.cpp.enableRtti;
-    if (enableRtti !== undefined && (tag === "cpp" || tag === "objcpp")) {
+    if (enableRtti !== undefined && (tag === "cpp" || tag === "objcpp" || tag === "cppm")) {
         args.push(enableRtti ? "-frtti" : "-fno-rtti");
     }
 
@@ -906,6 +907,7 @@ function compilerFlags(project, product, input, output, explicitlyDependsOn) {
             var knownValues = ["c2x", "c17", "c11", "c99", "c90", "c89"];
             return Cpp.languageVersion(input.cpp.cLanguageVersion, knownValues, "C");
         case "cpp":
+        case "cppm":
         case "objcpp":
             knownValues = ["c++23", "c++2b", "c++20", "c++2a", "c++17", "c++1z",
                            "c++14", "c++1y", "c++11", "c++0x",
@@ -922,7 +924,7 @@ function compilerFlags(project, product, input, output, explicitlyDependsOn) {
                                                            product.cpp.useLanguageVersionFallback));
     }
 
-    if (tag === "cpp" || tag === "objcpp") {
+    if (tag === "cpp" || tag === "objcpp" || tag === "cppm") {
         var cxxStandardLibrary = product.cpp.cxxStandardLibrary;
         if (cxxStandardLibrary && product.qbs.toolchain.includes("clang")) {
             args.push("-stdlib=" + cxxStandardLibrary);
@@ -952,6 +954,8 @@ function languageName(fileTag) {
         return 'c';
     else if (fileTag === 'cpp')
         return 'c++';
+    else if (fileTag === 'cppm')
+        return 'c++';
     else if (fileTag === 'objc')
         return 'objective-c';
     else if (fileTag === 'objcpp')
@@ -998,6 +1002,8 @@ function compilerEnvVars(config, compilerInfo)
         list.push("C_INCLUDE_PATH");
     else if (compilerInfo.tag === "cpp")
         list.push("CPLUS_INCLUDE_PATH");
+    else if (compilerInfo.tag === "cppm")
+        list.push("CPLUS_INCLUDE_PATH");
     else if (compilerInfo.tag === "objc")
         list.push("OBJC_INCLUDE_PATH");
     else if (compilerInfo.tag === "objccpp")
@@ -1031,17 +1037,21 @@ function linkerEnvVars(config, inputs)
 
 function setResponseFileThreshold(command, product)
 {
-    if (product.qbs.targetOS.includes("windows") && Host.os().includes("windows"))
-        command.responseFileThreshold = 10000;
+    if (Host.os().includes("windows"))
+        command.responseFileThreshold = 8000;
 }
 
-function prepareCompiler(project, product, inputs, outputs, input, output, explicitlyDependsOn) {
+function prepareCompilerInternal(project, product, inputs, outputs, input, output_, explicitlyDependsOn) {
+    var output = output_ || outputs["obj"][0];
+
     var compilerInfo = effectiveCompilerInfo(product.qbs.toolchain,
                                              input, output);
+
     var compilerPath = compilerInfo.path;
     var pchOutput = output.fileTags.includes(compilerInfo.tag + "_pch");
 
-    var args = compilerFlags(project, product, input, output, explicitlyDependsOn);
+    var args = compilerFlags(project, product, outputs, input, output, explicitlyDependsOn);
+
     var wrapperArgsLength = 0;
     var wrapperArgs = product.cpp.compilerWrapper;
     var extraEnv;
@@ -1084,6 +1094,13 @@ function prepareCompiler(project, product, inputs, outputs, input, output, expli
     return cmd;
 }
 
+function prepareCompiler(project, product, inputs, outputs, input, output, explicitlyDependsOn) {
+    var result = Cpp.prepareModules(project, product, inputs, outputs, input, output);
+    result = result.concat(prepareCompilerInternal(
+        project, product, inputs, outputs, input, output, explicitlyDependsOn));
+    return result;
+}
+
 // Concatenates two arrays of library names and preserves the dependency order that ld needs.
 function concatLibs(libs, deplibs) {
     var r = [];
@@ -1268,7 +1285,7 @@ function separateDebugInfoCommands(product, outputs, primaryOutput) {
     var debugInfo = outputs.debuginfo_app || outputs.debuginfo_dll
             || outputs.debuginfo_loadablemodule;
 
-    if (debugInfo) {
+    if (debugInfo && !product.qbs.toolchain.includes("emscripten")) {
         var objcopy = product.cpp.objcopyPath;
 
         var cmd = new Command(objcopy, ["--only-keep-debug", primaryOutput.filePath,
@@ -1315,6 +1332,43 @@ function separateDebugInfoCommandsDarwin(product, outputs, primaryOutputs) {
     return commands;
 }
 
+function librarySymlinkArtifacts(product, buildVariantSuffix) {
+    var artifacts = [];
+    if (product.cpp.shouldCreateSymlinks && (!product.bundle || !product.bundle.isBundle)) {
+        var maxVersionParts = product.cpp.internalVersion ? 3 : 1;
+        var primaryFileName =
+            PathTools.dynamicLibraryFilePath(product, buildVariantSuffix, undefined);
+        for (var i = 0; i < maxVersionParts; ++i) {
+            var symlink = {
+                filePath: FileInfo.joinPaths(product.destinationDirectory,
+                                             PathTools.dynamicLibraryFilePath(
+                                                 product, buildVariantSuffix, undefined, i)),
+                fileTags: ["dynamiclibrary_symlink"],
+                cpp: { primaryFileName: primaryFileName }
+            };
+            if (i > 0 && artifacts[i-1].filePath == symlink.filePath)
+                break; // Version number has less than three components.
+            artifacts.push(symlink);
+        }
+    }
+    return artifacts;
+}
+
+function librarySymlinkCommands(outputs, primaryOutput) {
+    var commands = [];
+    // Create symlinks from {libfoo, libfoo.1, libfoo.1.0} to libfoo.1.0.0
+    var links = outputs["dynamiclibrary_symlink"];
+    var symlinkCount = links ? links.length : 0;
+    for (i = 0; i < symlinkCount; ++i) {
+        var cmd = new Command("ln", ["-sf", links[i].cpp.primaryFileName, links[i].filePath]);
+        cmd.highlight = "filegen";
+        cmd.description = "creating symbolic link '" + links[i].fileName + "'";
+        cmd.workingDirectory = FileInfo.path(primaryOutput.filePath);
+        commands.push(cmd);
+    }
+    return commands;
+}
+
 function prepareLinker(project, product, inputs, outputs, input, output) {
     var i, primaryOutput, cmd, commands = [];
 
@@ -1361,7 +1415,10 @@ function prepareLinker(project, product, inputs, outputs, input, output) {
     setResponseFileThreshold(cmd, product);
     commands.push(cmd);
 
-    if (product.qbs.targetOS.includes("darwin")) {
+    if (product.qbs.toolchain.includes("emscripten") && outputs.application
+            && product.cpp.separateDebugInformation) {
+        args.push("-gseparate-dwarf");
+    } else if (product.qbs.targetOS.includes("darwin")) {
         if (!product.aggregate) {
             commands = commands.concat(separateDebugInfoCommandsDarwin(
                                            product, outputs, [primaryOutput]));
@@ -1372,19 +1429,7 @@ function prepareLinker(project, product, inputs, outputs, input, output) {
 
     if (outputs.dynamiclibrary) {
         Array.prototype.push.apply(commands, createSymbolCheckingCommands(product, outputs));
-
-        // Create symlinks from {libfoo, libfoo.1, libfoo.1.0} to libfoo.1.0.0
-        var links = outputs["dynamiclibrary_symlink"];
-        var symlinkCount = links ? links.length : 0;
-        for (i = 0; i < symlinkCount; ++i) {
-            cmd = new Command("ln", ["-sf", primaryOutput.fileName,
-                                     links[i].filePath]);
-            cmd.highlight = "filegen";
-            cmd.description = "creating symbolic link '"
-                    + links[i].fileName + "'";
-            cmd.workingDirectory = FileInfo.path(primaryOutput.filePath);
-            commands.push(cmd);
-        }
+        Array.prototype.push.apply(commands, librarySymlinkCommands(outputs, primaryOutput));
     }
 
     if (product.cpp.shouldSignArtifacts) {
@@ -1412,7 +1457,10 @@ function debugInfoArtifacts(product, variants, debugInfoTagSuffix) {
     variants = variants || [{}];
 
     var artifacts = [];
-    if (product.cpp.separateDebugInformation) {
+    var separateDebugInfo = product.cpp.separateDebugInformation;
+    if (separateDebugInfo && product.qbs.toolchain.includes("emscripten"))
+        separateDebugInfo = fileTag === "application";
+    if (separateDebugInfo) {
         variants.map(function (variant) {
             artifacts.push({
                 filePath: FileInfo.joinPaths(product.destinationDirectory,
@@ -1580,3 +1628,228 @@ function pathPrefix(baseDir, prefix)
         path += prefix;
     return path;
 }
+
+function appLinkerOutputArtifacts(product)
+{
+    var app = {
+        filePath: FileInfo.joinPaths(product.destinationDirectory,
+                                     PathTools.applicationFilePath(product)),
+        fileTags: ["bundle.input", "application"]
+            .concat(product.cpp.isForMainBundle ? ["bundle.main.input", "bundle.main.executable"] : [])
+            .concat(product.cpp.shouldSignArtifacts ? ["codesign.signed_artifact"] : []),
+        bundle: {
+            _bundleFilePath: FileInfo.joinPaths(product.destinationDirectory,
+                                                PathTools.bundleExecutableFilePath(product))
+        }
+    }
+    var artifacts = [app];
+    if (!product.aggregate)
+        artifacts = artifacts.concat(debugInfoArtifacts(product, undefined, "app"));
+    if (product.cpp.generateLinkerMapFile) {
+        artifacts.push({
+            filePath: FileInfo.joinPaths(product.destinationDirectory,
+                                         product.targetName + product.cpp.linkerMapSuffix),
+            fileTags: ["mem_map"]
+        });
+    }
+    if (product.qbs.toolchain.includes("emscripten"))
+        artifacts = artifacts.concat(wasmArtifacts(product));
+    return artifacts;
+}
+
+function moduleLinkerOutputArtifacts(product, inputs)
+{
+    var app = {
+        filePath: FileInfo.joinPaths(product.destinationDirectory,
+                                     PathTools.loadableModuleFilePath(product)),
+        fileTags: ["bundle.input", "loadablemodule"]
+            .concat(product.cpp.isForMainBundle ? ["bundle.main.input", "bundle.main.plugin"] : [])
+            .concat(product.cpp.shouldSignArtifacts ? ["codesign.signed_artifact"] : []),
+        bundle: {
+            _bundleFilePath: FileInfo.joinPaths(product.destinationDirectory,
+                                                PathTools.bundleExecutableFilePath(product))
+        }
+    }
+    var artifacts = [app];
+    if (!product.aggregate)
+        artifacts = artifacts.concat(debugInfoArtifacts(product, undefined,
+                                                        "loadablemodule"));
+    return artifacts;
+}
+
+function staticLibLinkerOutputArtifacts(product)
+{
+    var tags = ["bundle.input", "staticlibrary"]
+        .concat(product.cpp.isForMainBundle ? ["bundle.main.input", "bundle.main.library"] : [])
+        .concat(product.cpp.shouldSignArtifacts ? ["codesign.signed_artifact"] : []);
+    var objs = inputs["obj"];
+    var objCount = objs ? objs.length : 0;
+    for (var i = 0; i < objCount; ++i) {
+        var ft = objs[i].fileTags;
+        if (ft.includes("c_obj"))
+            tags.push("c_staticlibrary");
+        if (ft.includes("cpp_obj"))
+            tags.push("cpp_staticlibrary");
+    }
+    return [{
+        filePath: FileInfo.joinPaths(product.destinationDirectory,
+                                     PathTools.staticLibraryFilePath(product)),
+        fileTags: tags,
+        bundle: {
+            _bundleFilePath: FileInfo.joinPaths(product.destinationDirectory,
+                                                PathTools.bundleExecutableFilePath(product))
+        }
+    }];
+}
+
+function staticLibLinkerCommands(project, product, inputs, outputs, input, output,
+                                 explicitlyDependsOn)
+{
+    var commands = [];
+    var args = ['rcs'];
+    Array.prototype.push.apply(args, product.cpp.archiverFlags);
+    args.push(output.filePath);
+    for (var i in inputs.obj)
+        args.push(inputs.obj[i].filePath);
+    for (var i in inputs.res)
+        args.push(inputs.res[i].filePath);
+    var cmd = new Command(product.cpp.archiverPath, args);
+    cmd.description = 'creating ' + output.fileName;
+    cmd.highlight = 'linker'
+    cmd.jobPool = "linker";
+    cmd.responseFileUsagePrefix = '@';
+    setResponseFileThreshold(cmd, product);
+    commands.push(cmd);
+
+    if (product.cpp.shouldSignArtifacts) {
+        Array.prototype.push.apply(
+            commands, Codesign.prepareSign(project, product, inputs, outputs, input, output));
+    }
+    return commands;
+}
+
+function dynamicLibLinkerOutputArtifacts(product)
+{
+    var artifacts = [{
+        filePath: FileInfo.joinPaths(product.destinationDirectory,
+                                     PathTools.dynamicLibraryFilePath(product)),
+        fileTags: ["bundle.input", "dynamiclibrary"]
+            .concat(product.cpp.isForMainBundle ? ["bundle.main.input", "bundle.main.library"] : [])
+            .concat(product.cpp.shouldSignArtifacts ? ["codesign.signed_artifact"] : []),
+        bundle: {
+            _bundleFilePath: FileInfo.joinPaths(product.destinationDirectory,
+                                                PathTools.bundleExecutableFilePath(product))
+        }
+    }];
+    if (product.cpp.imageFormat === "pe") {
+        artifacts.push({
+            fileTags: ["dynamiclibrary_import"],
+            filePath: FileInfo.joinPaths(product.destinationDirectory,
+                                         PathTools.importLibraryFilePath(product)),
+            alwaysUpdated: false
+        });
+    } else {
+        // List of libfoo's public symbols for smart re-linking.
+        artifacts.push({
+            filePath: product.destinationDirectory + "/.sosymbols/"
+                      + PathTools.dynamicLibraryFilePath(product),
+            fileTags: ["dynamiclibrary_symbols"],
+            alwaysUpdated: false,
+        });
+    }
+
+    artifacts = artifacts.concat(librarySymlinkArtifacts(product));
+    if (!product.aggregate)
+        artifacts = artifacts.concat(debugInfoArtifacts(product, undefined, "dll"));
+    return artifacts;
+}
+
+function wasmArtifacts(product)
+{
+    var flags = product.cpp.driverLinkerFlags;
+    var wasmoption = 1;
+    var pthread = false;
+    for (var index in flags) {
+        var option = flags[index];
+        if (option.indexOf("WASM") !== -1) {
+            option = option.trim();
+            wasmoption = option.substring(option.length - 1);
+        } else if (option.indexOf("-pthread") !== -1) {
+            pthread = true;
+        }
+    }
+
+    var artifacts = [];
+    var createAppArtifact = function(fileName) {
+        return {
+            filePath: FileInfo.joinPaths(product.destinationDirectory, fileName),
+            fileTags: ["wasm"]
+        };
+    };
+    var suffix = product.cpp.executableSuffix;
+    if (suffix !== ".wasm") {
+        if (suffix === ".html")
+            artifacts.push(createAppArtifact(product.targetName + ".js"));
+        if (pthread)
+            artifacts.push(createAppArtifact(product.targetName + ".worker.js"));
+    }
+
+    if (wasmoption !== 0 && suffix !== ".wasm") //suffix .wasm will already result in "application".wasm
+        artifacts.push(createAppArtifact(product.targetName + ".wasm"));
+    if (wasmoption == 2)
+        artifacts.push(createAppArtifact(product.targetName + ".wasm.js"));
+
+      return artifacts;
+}
+
+function configureStdModules() {
+    try {
+        var probe = new Process();
+        // check which stdlib is used? this actually works:
+        // clang-18 -print-file-name=libstdc++.modules.json --sysroot /opt/gcc-15/
+        const modulesJsonFiles = [
+            "libc++.modules.json",
+            "libstdc++.modules.json",
+            "../../c++/libc++.modules.json",
+        ];
+        for (var i = 0; i < modulesJsonFiles.length; ++i) {
+            const modulesJsonFile = modulesJsonFiles[i];
+            const result = probe.exec(_compilerPath, ["-print-file-name=" + modulesJsonFile], false);
+            if (result !== 0)
+                continue;
+            const path = probe.readStdOut().trim();
+            if (path !== modulesJsonFile && FileInfo.isAbsolutePath(path) && File.exists(path)) {
+                const jsonFile = new TextFile(path, TextFile.ReadOnly);
+
+                const json = JSON.parse(jsonFile.readAll());
+                jsonFile.close();
+                const modules = json.modules
+                    .filter(function(module) {
+                        const logicalName = module["logical-name"];
+                        if (logicalName === "std" && (_forceUseImportStd || _forceUseImportStdCompat)) {
+                            return true;
+                        } else if (logicalName === "std.compat" && _forceUseImportStdCompat) {
+                            return true;
+                        }
+                        return false;
+                    })
+                    .map(function(module) {
+                        const sourcePath = module["source-path"];
+                        if (FileInfo.isAbsolutePath(sourcePath))
+                            return sourcePath;
+                        return FileInfo.joinPaths(FileInfo.path(path), sourcePath);
+                    })
+                    .filter(function(module) {
+                        return File.exists(module);
+                    });
+                if (modules.length > 0) {
+                    found = true;
+                    _stdModulesFiles = modules;
+                    break;
+                }
+            }
+        }
+    } finally {
+        probe.close();
+    }
+}
