@@ -199,17 +199,18 @@ QString ResolveCommand::representation() const
 
 static QList<CommandLineOption::Type> resolveOptions()
 {
-    return {CommandLineOption::FileOptionType,
-            CommandLineOption::BuildDirectoryOptionType,
-            CommandLineOption::LogLevelOptionType,
-            CommandLineOption::VerboseOptionType,
-            CommandLineOption::QuietOptionType,
-            CommandLineOption::ShowProgressOptionType,
-            CommandLineOption::DryRunOptionType,
-            CommandLineOption::ForceProbesOptionType,
-            CommandLineOption::LogTimeOptionType,
-            CommandLineOption::DeprecationWarningsOptionType,
-            CommandLineOption::DisableFallbackProviderType};
+    return {
+        CommandLineOption::FileOptionType,
+        CommandLineOption::BuildDirectoryOptionType,
+        CommandLineOption::LogLevelOptionType,
+        CommandLineOption::VerboseOptionType,
+        CommandLineOption::QuietOptionType,
+        CommandLineOption::ShowProgressOptionType,
+        CommandLineOption::DryRunOptionType,
+        CommandLineOption::ForceProbesOptionType,
+        CommandLineOption::LogTimeOptionType,
+        CommandLineOption::DeprecationWarningsOptionType,
+        CommandLineOption::JobsOptionType};
 }
 
 QList<CommandLineOption::Type> ResolveCommand::supportedOptions() const
@@ -279,7 +280,6 @@ static QList<CommandLineOption::Type> buildOptions()
             << CommandLineOption::ForceTimestampCheckOptionType
             << CommandLineOption::ForceOutputCheckOptionType
             << CommandLineOption::BuildNonDefaultOptionType
-            << CommandLineOption::JobsOptionType
             << CommandLineOption::CommandEchoModeOptionType
             << CommandLineOption::NoInstallOptionType
             << CommandLineOption::RemoveFirstOptionType
