@@ -285,6 +285,11 @@ ArtifactData ProjectPrivate::createArtifactData(const Artifact *artifact,
     ta.d->filePath = artifact->filePath();
     ta.d->fileTags = artifact->fileTags().toStringList();
     ta.d->properties.d->m_map = artifact->properties;
+    for (const Artifact *a : filterByType<Artifact>(artifact->children)) {
+        ta.d->childPaths << a->filePath();
+        for (const auto *f : a->fileDependencies)
+            ta.d->childPaths << f->filePath();
+    }
     ta.d->isGenerated = artifact->artifactType == Artifact::Generated;
     ta.d->isTargetArtifact = targetArtifacts.contains(const_cast<Artifact *>(artifact));
     ta.d->isValid = true;
@@ -460,6 +465,25 @@ void ProjectPrivate::addFiles(const ProductData &product, const GroupData &group
     adder.apply();
 }
 
+void ProjectPrivate::addDependencies(
+    const ProductData &product, const GroupData &group, const QStringList &dependencies)
+{
+    GroupUpdateContext groupContext = getGroupContext(product, group);
+    QStringList depsToInsert = dependencies;
+    for (auto it = depsToInsert.begin(); it != depsToInsert.end();) {
+        if (product.dependencies().contains(*it))
+            it = depsToInsert.erase(it);
+        else
+            ++it;
+    }
+
+    ProjectFileDependenciesAdder adder(
+        groupContext.products.front(),
+        group.isValid() ? groupContext.groups.front() : GroupData(),
+        depsToInsert);
+    adder.apply();
+}
+
 void ProjectPrivate::removeFiles(const ProductData &product, const GroupData &group,
                                  const QStringList &filePaths)
 {
@@ -637,6 +661,13 @@ void ProjectPrivate::retrieveProjectData(ProjectData &projectData,
         product.d->isMultiplexed = productIsMultiplexed(resolvedProduct);
         product.d->properties = resolvedProduct->productProperties;
         product.d->moduleProperties.d->m_map = resolvedProduct->moduleProperties;
+        for (const ResolvedModulePtr &m : std::as_const(resolvedProduct->modules)) {
+            ModuleData module;
+            module.d->name = m->name;
+            module.d->location = m->location;
+            module.d->properties = m->properties;
+            product.d->modules << module;
+        }
         for (const GroupPtr &resolvedGroup : resolvedProduct->groups) {
             if (resolvedGroup->targetOfModule.isEmpty() && resolvedGroup->files) {
                 product.d->groups << createGroupDataFromGroup(resolvedGroup, resolvedProduct);
@@ -659,6 +690,14 @@ void ProjectPrivate::retrieveProjectData(ProjectData &projectData,
                 ta.d->filePath = it.key();
                 ta.d->fileTags = it.value().fileTags.toStringList();
                 ta.d->properties.d->m_map = it.value().properties;
+                Artifact * const child = lookupArtifact(resolvedProduct, it.key(), true);
+                if (child) {
+                    for (const Artifact *a : filterByType<Artifact>(child->children)) {
+                        ta.d->childPaths << a->filePath();
+                        for (const auto *f : a->fileDependencies)
+                            ta.d->childPaths << f->filePath();
+                    }
+                }
                 ta.d->isGenerated = true;
                 ta.d->isTargetArtifact = resolvedProduct->fileTags.intersects(it.value().fileTags);
                 ta.d->isValid = true;
@@ -666,10 +705,8 @@ void ProjectPrivate::retrieveProjectData(ProjectData &projectData,
                 product.d->generatedArtifacts << ta;
             }
         }
-        for (const ResolvedProductPtr &resolvedDependentProduct
-             : std::as_const(resolvedProduct->dependencies)) {
-            product.d->dependencies << resolvedDependentProduct->fullDisplayName();
-        }
+        for (const ProductDependency &dep : std::as_const(resolvedProduct->dependencies))
+            product.d->dependencies << dep.product->fullDisplayName();
         std::sort(product.d->type.begin(), product.d->type.end());
         std::sort(product.d->groups.begin(), product.d->groups.end());
         std::sort(product.d->generatedArtifacts.begin(), product.d->generatedArtifacts.end());
@@ -1090,6 +1127,31 @@ ErrorInfo Project::addFiles(const ProductData &product, const GroupData &group,
     }
 }
 
+/*!
+ * \brief Adds \c Depends items for the given dependencies to the given product.
+ * If \c group is a default-constructed object, the items will be added at the top level,
+ * otherwise to \c group.
+ * After calling this function, it is recommended to re-fetch the project data, as other
+ * items can be affected.
+ * \sa qbs::Project::projectData()
+ */
+ErrorInfo Project::addDependencies(
+    const ProductData &product, const GroupData &group, const QStringList &dependencies)
+{
+    try {
+        QBS_CHECK(isValid());
+        d->prepareChangeToProject();
+        d->addDependencies(product, group, dependencies);
+        d->internalProject->store(d->logger);
+        return {};
+    } catch (const ErrorInfo &exception) {
+        auto errorInfo = exception;
+        errorInfo.prepend(
+            Tr::tr("Failure adding depencencies to product '%1'.").arg(product.name()));
+        return errorInfo;
+    }
+}
+
 /*!
  * \brief Removes the given files from the given product.
  * If \c group is a default-constructed object, the files will be removed from the product's
