@@ -40,18 +40,18 @@
 #include "error.h"
 
 #include "persistence.h"
-#include "qttools.h"
-#include "stringconstants.h"
 #include "setupprojectparameters.h"
-#include "logging/logger.h"
+#include "stringconstants.h"
+
+#include <logging/logger.h>
+#include <logging/translator.h>
+#include <tools/stlutils.h>
 
 #include <QtCore/qjsonarray.h>
 #include <QtCore/qjsonobject.h>
 #include <QtCore/qshareddata.h>
 #include <QtCore/qstringlist.h>
 
-#include <algorithm>
-#include <functional>
 #include <regex>
 
 namespace qbs {
@@ -207,15 +207,22 @@ ErrorInfo::ErrorInfo(const QString &description, const QStringList &backtrace)
 {
     append(description);
     for (const QString &traceLine : backtrace) {
-        static const std::regex regexp("^(.+) at (.+):(\\-?[0-9]+)$");
+        if (traceLine.contains(QStringLiteral("<eval>")))
+            continue;
+        static const std::regex regexpWithFunc(
+            "^(.+) at [^(]*\\((.+):(\\-?[0-9]+):(\\-?[0-9]+)\\)$");
+        static const std::regex regexpWithoutFunc("^(.+) at (.+):(\\-?[0-9]+):(\\-?[0-9]+)$");
         std::smatch match;
         const std::string tl = traceLine.toStdString();
-        if (std::regex_match(tl, match, regexp)) {
-            const QString message = QString::fromStdString(match[1]),
-                             file = QString::fromStdString(match[2]),
-                             line = QString::fromStdString(match[3]);
+        bool hasMatch = std::regex_match(tl, match, regexpWithFunc);
+        if (!hasMatch)
+            hasMatch = std::regex_match(tl, match, regexpWithoutFunc);
+        if (hasMatch) {
+            const QString message = QString::fromStdString(match[1]).trimmed();
+            const QString file = QString::fromStdString(match[2]);
+            const QString line = QString::fromStdString(match[3]);
             const CodeLocation location(file, line.toInt());
-            appendBacktrace(message, location);
+            appendBacktrace(message, CodeLocation(file, line.toInt()));
         }
     }
 }
@@ -246,6 +253,19 @@ void ErrorInfo::prepend(const QString &description, const CodeLocation &location
     d->items.prepend(ErrorItem(description, location));
 }
 
+void ErrorInfo::addOrPrependLocation(const CodeLocation &loc)
+{
+    QBS_CHECK(hasError());
+    if (!loc.isValid())
+        return;
+    if (d->items.first().codeLocation() == loc)
+        return;
+    if (d->items.first().codeLocation().isValid())
+        prepend(Internal::Tr::tr("While evaluating here"), loc);
+    else
+        d->items[0] = ErrorItem(d->items.first().description(), loc);
+}
+
 /*!
  * \brief A list of concrete error description.
  * Most often, there will be one element in this list, but there can be more e.g. to illustrate
@@ -269,7 +289,7 @@ void ErrorInfo::clear()
 QString ErrorInfo::toString() const
 {
     QStringList lines;
-    for (const ErrorItem &e : qAsConst(d->items)) {
+    for (const ErrorItem &e : std::as_const(d->items)) {
         if (e.isBacktraceItem()) {
             QString line;
             if (!e.description().isEmpty())
@@ -306,8 +326,15 @@ bool ErrorInfo::isInternalError() const
 
 bool ErrorInfo::hasLocation() const
 {
-    return std::any_of(d->items.cbegin(), d->items.cend(),
-                        [](const ErrorItem &ei) { return ei.codeLocation().isValid(); });
+    return Internal::any_of(d->items, [](const ErrorItem &ei) {
+        return ei.codeLocation().isValid(); });
+}
+
+bool ErrorInfo::isCancelException() const
+{
+    return Internal::any_of(d->items, [](const ErrorItem &ei) {
+        return ei.description() == QLatin1String("interrupted");
+    });
 }
 
 void ErrorInfo::load(Internal::PersistentPool &pool)
@@ -332,7 +359,7 @@ void handlePropertyError(
 {
     if (params.propertyCheckingMode() == ErrorHandlingMode::Strict)
         throw error;
-    logger.printWarning(error);
+    logger.printError(error);
 }
 
 } // namespace qbs
