@@ -44,11 +44,10 @@
 #include <QtCore/qshareddata.h>
 #include <QtCore/qmetatype.h>
 #include <QtCore/qprocess.h>
+#include <QtCore/qstringlist.h>
 
 QT_BEGIN_NAMESPACE
 class QJsonObject;
-class QString;
-class QStringList;
 QT_END_NAMESPACE
 
 namespace qbs {
